<?php

//
// innovaphone fax PHP class
// copyright (c) innovaphone 2012 - 2015
//
// extends class InnovaphonePbxInfo
//

require_once('innovaphone.pbx.info.class.php');
require_once('innovaphone.fax.call.class.php');
require_once('innovaphone.fax.shm.class.php');

class InnovaphoneFax extends InnovaphonePbxInfo {

    protected $userName;

    // class constructor
    public function __construct(InnovaphonePbx &$pbx, $userName, &$webdavUrl) {
        parent::__construct($pbx);
        $this->userName     = $userName;
    }
    
    public function updateCalls(&$callInfo, &$faxShm = null) {
        foreach($callInfo as &$ci) {
            if(!array_key_exists($ci->call, $this->calls)) {
                $this->calls[$ci->call] = new InnovaphoneFaxCall();
            }
            $update = $this->calls[$ci->call]->update($ci);
            if($update && $faxShm) {
                $faxShm->updateCall($this->calls[$ci->call]);
            }
        }
    }

    public function makeCall($userHandle, $sendNumber, $destinationNumber) {
        $callHandle = 0;
        $user       = $this->getUser($this->userName);
        $info       = array(array("type" => "audio", "vali" => "", "vals" => ""));
        
        try {
            $callHandle = $this->pbx->UserCall($userHandle, null, $destinationNumber, null, 0, $info, 0, $sendNumber);
        }
        catch(Exception $e) {
        }
        return $callHandle;
    }

    public function getCall($callHandle) {
        if(array_key_exists($callHandle, $this->calls)) {
            return $this->calls[$callHandle];
        }
        return false;
    }

    public function findCall($confId) {
        foreach($this->calls as &$call) {
            if($call->getConfId() === $confId) {
                return $call;
            }
        }
        return false;
    }

    public function callInProgress($callHandle) {
        $inProgress = false;
        $call = $this->getCall($callHandle);
        
        if($call) {
            $state      = $call->getState() & 0xF;
            $inProgress = $state > 5 ? false : true;
        }
        return $inProgress;
    }

    public function updateOutgoingCall($callHandle, $webdavFileName, $subscriberId = "", $headerLine = "", $trace = false, $modem = "") {
        $update = false;
        $call = $this->getCall($callHandle);

        if($call) {
            $state      = $call->getState() & 0xF;
            $lastState  = $call->getLastState() & 0xF;
            if($state > $lastState) {
                if($state == 4) {
                    $update = true;
                    //print "Alerting.\n";
                }
                else if($state == 5) {
                    $update = true;
                    //print "Connected.\n";
                    print "Using file '" . $webdavFileName;
                    $call->setWebdavFileName($webdavFileName);
                    $uui = $webdavFileName . '?direction=outgoing';
                    if($trace) {
                        $uui .= "&trace=true";
                    }
                    if($modem != "") {
                        $uui .= "&modem=" . $modem;
                    }
                    if($subscriberId != "")  {
                        print "'\n and subscriber id '" . $subscriberId;
                        $uui .= "&subscriber-id=$subscriberId";
                    }
                    if($headerLine != "") {
                        print "'\n and header line '" . $headerLine;
                        $uui .= "&header-line=$headerLine";
                    }
                    print "'.\n";
                    try {
                        $this->pbx->UserUUI($callHandle, 'true', "?");
                        $this->pbx->UserUUI($callHandle, 'true', $uui);
                    }
                    catch (Exception $e) {
                        print "SOAP connection error, cannot set header line.";
                    }                    
                }
                else if($state > 5 && $call->getFaxState() != "completed") {
                    $update = true;
                    //printf("Closed (0x%x).\n", $call->getCauseCode());
                }
            }
            else if($call->getFaxState() != $call->getLastFaxState()) {
                $update = true;
            }
            $call->setLastState($state);
            $call->setLastFaxState($call->getFaxState());
            //print "Fax state: " . $call->getFaxState() . "\n";
        }
        return $update;
    }

    public function handleNewIncomingCalls($userHandle, $configurationId, $userNumberPrefix, &$faxShm = null) {
        $user       = $this->getUser($this->userName);
        $callList   = $this->findUserCalls($userHandle, false);
        
        foreach($callList as &$call) {
            $faxState       = $call->getFaxState();
            if($faxState == "waiting") {
                $incomplete         = true;
                $calledNumber       = $call->getCalledNumber();
                $destinationUser    = $this->findDestinationUser($call->getUserHandle(), $calledNumber, $userNumberPrefix, $incomplete, $call->getSendingComplete());
                if(!$incomplete) {
                    if($destinationUser) {
                        $destinationUserH323    = $destinationUser->getH323();
                        $destinationUserNumber  = $destinationUser->getNumber();
                        $call->setAssigned('');
                        if($faxShm) {
                            $faxShm->updateCall($call);
                        }
                        print getLogDate() . "Receive fax from '" . $call->getCallingNumber() . "' to '" . $destinationUserH323 . "'.\n";
                        $path = dirname(__FILE__);
                        $script = $path.'/../scripts/innovaphone.fax.call.in.php';
                        $log    = '>> ' . escapeshellarg(FaxConfig::logPath.'/' . $configurationId . '-call-in-' . $call->getConfId() . '.log') . ' 2>&1';
                        $exec = "/usr/bin/php $script \"$configurationId\" \"{$call->getGuid()}\" $log &";
                        //print "start: " . $exec;
                        exec($exec);
                    }
                    else {
                        try {
                            $this->pbx->UserClear($call->getCallHandle(), 0x1/*Q931_CAUSE_UnallocatedNumber_B_TinNr*/, array());
                        }
                        catch (Exception $e) {
                        }
                    }
                }
            }
            if(!$call->isActive() && $faxShm) {
                $faxShm->updateCall($call);
            }
        }
    }

    public function handleIncomingCall($callHandle, $webdavFileName, $subscriberId, $userNumberPrefix, $clear = false, $trace = false, $modem = "") { 
        $call = $this->getCall($callHandle);

        if($call) {
            $callState      = $call->getState();
            if(($callState & 0xf) <= 4/*Alert*/ && $clear == false) {
                $incomplete         = true;
                $calledNumber       = $call->getCalledNumber();
                $destinationUser    = $this->findDestinationUser($call->getUserHandle(), $calledNumber, $userNumberPrefix, $incomplete, $call->getSendingComplete());
                if(!$incomplete) {
                    if($destinationUser) {
                        print "Using file '" . $webdavFileName;
                        $uui = $webdavFileName . '?direction=incoming';
                        if($trace) {
                            $uui .= "&trace=true";
                        }
                        if($modem != "") {
                            $uui .= "&modem=" . $modem;
                        }
                        if($subscriberId != "") {
                            print "'\n and subscriber id '" . $subscriberId;
                            $uui .= "&subscriber-id=$subscriberId";
                        }
                        print "'.\n";
                        try {
                            $this->pbx->UserUUI($call->getCallHandle(), 'true', "?");
                            $this->pbx->UserUUI($call->getCallHandle(), 'true', $uui);
                            $this->pbx->UserConnect($call->getCallHandle());
                        }
                        catch (Exception $e) {
                            die("SOAP connection error.");
                        }
                    }
                    else {
                        $clear = true;
                    }
                }
            }
            if($clear) {
                try {
                    $this->pbx->UserClear($call->getCallHandle(), 0x1/*Q931_CAUSE_UnallocatedNumber_B_TinNr*/, array());
                }
                catch (Exception $e) {
                }
            }
        }
    }

    public function showIncomingCalls($userHandle, $userNumberPrefix) { 
        $user       = $this->getUser($this->userName);
        $callList   = $this->findUserCalls($userHandle, false);

        if(count($callList)) {
            print "\nNo.     CGPN -> CDPN    Dest. user      State File name                    Code";
            print "\n===============================================================================\n";
        }

        foreach($callList as $key => &$call) {
            $callState          = $call->getState();
            $ownNumber          = $user->getNodeNumber() . $user->getNumber();
            $cgpn               = $call->getCallingNumber();
            $calledNumber       = $call->getCalledNumber();
            $callState          = $call->getState();
            $callStateName      = $call->getStateName();
            $incomplete         = true;
            $destinationNumber  = '';

            if(!strncmp($ownNumber, $cdpn, strlen($ownNumber))) {
                $destinationNumber = substr($cdpn, strlen($ownNumber));
            }

            $destinationUserName    = '';
            $destinationUserNumber  = '';
            $destinationUser        = $this->findDestinationUser($call->getUserHandle(), $calledNumber, $userNumberPrefix, $incomplete, $call->getSendingComplete());
            
            if(!$incomplete) {
                if($destinationUser) {
                    $destinationUserName    = $destinationUser->getName();
                    $destinationUserNumber  = $destinationUser->getNumber();
                }
                else {
                    $destinationUserName = '-';
                }
            }
            $fileName = $call->getWebdavFileName();
            if(strlen($fileName) > 28) {
                $fileName =  '..' . substr($fileName, -26);
            }
            printf("%2d: %3d %-7s %-7s %-10s %-4s 0x%03x %-28.28s 0x%-2x\n", 
                $key, $call->getCallHandle(), $cgpn, $calledNumber, 
                $destinationUserName, $destinationUserNumber, $callState, 
                $fileName, $call->getCauseCode());
        }
    }

    public function showOutgoingCalls($userHandle) {
        $callList   = $this->findUserCalls($userHandle, true);

        if(count($callList)) {
            print "\nNo.     CGPN -> CDPN  State                 Rel.code";
            print "\n==========================================================================\n";
        }

        foreach($callList as $key => &$call) {
            $cgpn           = $call->getCallingNumber();
            $cdpn           = $call->getCalledNumber();
            $callState      = $call->getState();
            $callStateName  = $call->getStateName();

            printf("%2d: %3d %-7s %-5s 0x%03x %-15s 0x%-2x\n", 
                $key, $call->getCallHandle(), $cgpn, $cdpn, $callState, $callStateName, $call->getCauseCode());
        }
    }

    public function findDestinationUser($userHandle, $calledNumber, $userNumberPrefix, &$incomplete, $sendingComplete = false) {
        $user               = $this->getUser($this->userName);
        $ownNumber          = $user->getNodeNumber() . $user->getNumber();
        $incomplete         = strlen($ownNumber) > strlen($calledNumber);
        $destinationUser    = 0;
 
        if(!strncmp($ownNumber, $calledNumber, strlen($ownNumber))) {
            $destinationNumber  = substr($calledNumber, strlen($ownNumber));
            $incomplete         = true;
            if($destinationNumber === false) {
                $destinationUser = $user;
            }
            else {
                $destinationUser = $this->findUserWithNumber($userHandle, $userNumberPrefix . $destinationNumber, $incomplete);
            }
            if(!$incomplete || $sendingComplete) {
                if(!$destinationUser && $userNumberPrefix != "") {
                    $destinationUser    = $this->findUserWithNumber($userHandle, $destinationNumber, $incomplete);
                    if($destinationUser && $destinationUser->getPseudoType != "map") {
                        $destinationUser = 0;
                    }
                }
                $incomplete = false;
            }
        }
        return $destinationUser;
    }
};

?>