<?php

//
// innovaphone incoming fax convert in PHP script
// copyright (c) innovaphone 2012 - 2014
//

$path=dirname(__FILE__);
require_once("$path/../include.php");
require_once("classes/translator.class.php");
require_once("classes/inputpage.class.php");
require_once("$path/../app-classes/innovaphone.fax.config.class.php");
require_once("$path/../app-classes/innovaphone.fax.database.class.php");
require_once("$path/../app-classes/innovaphone.fax.shm.class.php");
require_once("$path/../app-classes/innovaphone.pbx.class.php");
require_once("$path/../app-classes/innovaphone.pbx.info.class.php");

$version = file_get_contents("$path/../version");

declare(ticks = 1);
?>

innovaphone incoming fax convert in PHP script
copyright (c) innovaphone 2012 - 2014
<?php

print "Version " . $version . "\n\n";

$help = false;
foreach($argv as $arg) {
    if(in_array($arg, array('--help', '-help', '-h', '-?'))) {
        $help = true;
    }
}

if(!is_array($_GET) && ($argc < 3 || $help)) {
?>
    Usage:
  <?php echo $argv[0]; ?> <name> <jobGuid>

<id> is the configuration database id specific for this server instance.
<jobGuid> is the guid of the fax job which should be handled.

With the --help, -help, -h, or -? options, you can get this help.

<?php
exit(1);
} 

$configurationId        = null;
$jobGuid                = null;

if($argc >= 2) {
    $configurationId    = intval($argv[1]);
    $jobGuid            = $argv[2];
}
else if(is_array($_GET)) {
    $configurationId    = intval($_GET['id']);
    $jobGuid            = $_GET['jobGuid'];
}


setlocale(LC_CTYPE, "en_US.UTF-8");

function getLogDate() {
    $now = new DateTime();
    return date('Ymd-His') . "  -  ";
}

function sig_handler($nsig)
{
    print "\n";
    print getLogDate() . "terminated($nsig).\n";
    exit;
}

pcntl_signal(SIGTERM, "sig_handler", false);
pcntl_signal(SIGINT, "sig_handler", false);
pcntl_signal(SIGHUP, "sig_handler", false);


function shutdown_function() {
    global $pbx;
    global $faxShm;
    global $jobGuid;
    if($faxShm && $jobGuid) {
        $faxShm->unlockJob($jobGuid);
    }
    if($pbx && $pbx->getKey()) {
        terminateSession($pbx);
    }
    print "\n";
}

register_shutdown_function('shutdown_function');

$pbx        = null;
$faxShm     = null;


print getLogDate() . "Reading configuration... ";
$cfg     = new FaxServer();
$cfg->read("", $configurationId);
if(!$cfg) {
    print "failed.\n";
    die("Could not read configuration.");
}
print "done.\n";


tl::change("en");
tl::register("faxserver", dirname(__FILE__) . "/../lang");
tl::change($cfg->lang);


print getLogDate() . "Attaching to shared memory... ";
$faxShm     = new InnovaphoneFaxShm($cfg->shmId);
if(!$faxShm->isAttached()) {
    print "failed.\n";
    die("Could not attach to the SHM.");
}
print "done.\n";


print getLogDate() . "Trying to lock job... ";
$ret = $faxShm->lockJob($jobGuid, null, null);
if(!$ret) {
    $faxShm = 0;
    print "failed.\n";
    die("Could not lock fax job.");
}
print "done.\n";


print getLogDate() . "Connecting to database... ";
$faxDatabase    = new InnovaphoneFaxDatabase();
if(!$faxDatabase->isConnected()) {
    print "failed.\n";
    die("Could not connect to the database.");
}
$faxDatabase->setServerId($cfg->id);
print "done.\n";


print getLogDate() . "Reading fax job database entry... ";
$faxJob     = $faxDatabase->getFaxJob($jobGuid);
if(!$faxJob) {
    print "failed.\n";
    die("Could not read database object.");   
}
print "done.\n";


if($faxJob && $faxJob->getProgress() == 'closed') {
    die("Fax job is already closed.");
}
if($faxJob && $faxJob->getDirection() == 'outgoing') {
    die("Fax job has got wrong direction.");
}


function onConvertError(&$faxDatabase, &$faxJob, $error, $message = "", $to = "", $from = "") {
    if($faxDatabase && $faxJob) {
        print getLogDate() . "Updating database... ";
        $faxJob->setProgress('closed');
        $faxJob->setResult('error');
        $faxJob->setErrorCause('by system');
        $faxJob->setErrorDescription($error);
        $faxJob = $faxDatabase->updateFaxJob($faxJob);
        if(!$faxJob) {
            print "failed.\n";
            die("Could not update database object during convert error \"$error\"");   
        }
        print "done.\n";
    }
    die($error . "\n");
}


if($faxJob && $faxJob->getProgress() == 'queued') {

    print getLogDate() . "Converting fax document into TIFF format... ";

    setlocale(LC_CTYPE, "en_US.UTF-8");
    $fileWebdav         = $faxJob->getFileName();
    $fileData           = pathinfo($fileWebdav);
    $fileName           = $fileData['filename'];
    $filePath           = FaxConfig::documentPath;
    $fileWebdavLocal    = escapeshellarg(FaxConfig::webdavPath . "/" . $fileData['basename']);
    $fileConvert        = escapeshellarg($filePath . "/" . $fileData['basename']);
    $fileTiff           = escapeshellarg($filePath . "/" . $fileName . '.tif');
    $filePDF            = $filePath . "/" . $fileName . '.pdf';
    $fileConverted      = $filePath . "/" . $fileName . '.tif';

    $from               = imap_rfc822_write_address('faxserver', $cfg->faxServerDomain, $cfg->name);
    $to                 = '"' . mb_encode_mimeheader($faxJob->getUserName(), "UTF-8", "Q", "") . '" <' . $faxJob->getUserH323() . '>';

    if(true) {
        $fileConvert = $fileWebdavLocal;
    }
    else {
        $error      = "converting stopped, extern webdav not yet supported.";
        $message    = $error;
        onConvertError($faxDatabase, $faxJob, $error, $message, $to, $from);
    }

    $exec       = "sfftobmp -t $fileConvert -o $fileTiff 2>&1";
    $outputs    = array();
    $ret        = false;

    exec($exec, $outputs, $ret);

    if($ret) {
        $output     = "";
        foreach($outputs as &$row) {
            $output .= $row . "\n";
        }
        $message    = "Received fax document '$fileWebdav' could not be converted into TIFF.\n\nOutput:\n$output";
        $error      = "sfftobmp failed, exit code $ret";

        print "failed.\n";
        print "Command: " . $exec . "\n";
        print "Return code: " . $ret . "\n";
        print $message  . "\n";
        
        onConvertError($faxDatabase, $faxJob, $error, $message, $to, $from);
    }

    print "done.\n";

    
    if($cfg->faxForwardedFileFormat == Faxserver::fileFormatPDF) {
    
        print getLogDate() . "Converting fax document into PDF format... ";

        $maxPageRows    = 2308;

        $exec           = "tiffinfo $fileTiff | grep \"Image Width\" | cut -d \" \" -f8";
        $outputs        = array();
        $ret            = false;

        exec($exec, $outputs, $ret);
        if(!$ret) {
            foreach($outputs as &$row) {
                $maxPageRows = max($maxPageRows, intval($row));
            }                
        }

        // tiff2pdf -u m -w 21.0 -l 29.7 $fileTiff -o $fileConverted
        $fileConverted  = $filePDF;
        $fitPage        = ($maxPageRows > 2308) ? "-F" : "";
        $creator        = "\"innovaphone Faxserver\"";
        $author         = escapeshellarg($faxJob->getRemoteSubscriberId());
        $exec           = "tiff2pdf -pA4 $fitPage -c $creator -a $author $fileTiff > " . escapeshellarg($fileConverted);
        $outputs        = array();
        $ret            = false;

        exec($exec, $outputs, $ret);

        if($ret) {
            $output     = "";
            foreach($outputs as &$row) {
                $output .= $row . "\n";
            }
            $message    = "Received fax document '$fileWebdav' could not be converted into PDF.\n\nOutput:\n$output";
            $error      = "tiff2pdf failed, exit code $ret";
            
            print "failed.\n";
            print "Command: " . $exec . "\n";
            print "Return code: " . $ret . "\n";
            print $message  . "\n";

            onConvertError($faxDatabase, $faxJob, $error, $message, $to, $from);
        }

        $exec       = "rm -f $fileTiff 2>&1";
        $outputs    = array();
        $ret        = false;
        exec($exec, $outputs, $ret);

        print "done.\n";
    }

    print getLogDate() . "Updating database... ";

    $faxJob->setProgress('converted');
    $faxJob->setFileName($fileConverted);
    $faxJob = $faxDatabase->updateFaxJob($faxJob);
    if(!$faxJob || $faxJob->getProgress() != 'converted') {
        print "failed.\n";
        die("Could not update database object.");   
    }

    if(true) {
        $exec       = "rm -f $fileWebdavLocal 2>&1";
        $outputs    = array();
        $ret        = false;
        exec($exec, $outputs, $ret);
    }    
    
    print "done.\n";
}

$faxShm->unlockJob($jobGuid);

print getLogDate() . "Finished.\n";

?>
